package cd.casic.ci.process.process.service.report.impl;

import cd.casic.ci.process.dal.req.report.ReportDeleteReq;
import cd.casic.ci.process.dal.resp.report.ReportResp;
import cd.casic.ci.process.engine.context.ConstantContextHolder;
import cd.casic.ci.process.process.dataObject.base.BaseIdPageReq;
import cd.casic.ci.process.process.dataObject.base.BaseIdReq;
import cd.casic.ci.process.process.service.report.ReportService;
import cd.casic.framework.commons.exception.ServiceException;
import cd.casic.framework.commons.exception.enums.GlobalErrorCodeConstants;
import cd.casic.framework.commons.pojo.PageResult;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;

import java.security.KeyManagementException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.List;

import static cd.casic.ci.process.engine.worker.base.HttpWorker.getRestTemplateWithoutSANCheck;

/**
 * @author HopeLi
 * @version v1.0
 * @ClassName TargetVersionServiceImpl
 * @Date: 2025/5/17 15:20
 * @Description:
 */
@Service
@Slf4j
public class ReportServiceImpl implements ReportService {

    @Override
    public PageResult<ReportResp> findReportList(BaseIdPageReq req) throws NoSuchAlgorithmException, KeyStoreException, KeyManagementException {
        RestTemplate restTemplate = getRestTemplateWithoutSANCheck();
        String reportFindUrl = ConstantContextHolder.getScaIp() + "openapi/v1/asset/report/list";
        MultiValueMap<String, Object> body = buildFindRequestBody(req);
        HttpHeaders headers = createHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        headers.add("OpenApiUserToken", ConstantContextHolder.getScaToken());
        HttpEntity<MultiValueMap<String, Object>> requestEntity = new HttpEntity<>(body, headers);

        log.info("报告分页查询接口:" + reportFindUrl);
        JSONObject response = restTemplate.postForObject(reportFindUrl, requestEntity, JSONObject.class);
        String message = response.getString("message");
        if (message.equals("success")) {
            List<ReportResp> respList = new ArrayList<>();
            log.info("===============报告分页查询接口成功=================");
            JSONObject data = response.getJSONObject("data");
            setReportFindResp(data,response.getString("xmTraceId"),respList);

            PageResult<ReportResp> pageResult = new PageResult<>(respList,data.getLong("total"),data.getLong("pageNum"),data.getLong("pageSize"));
            return pageResult;

        } else {
            log.error("===============报告分页查询接口失败=================");
            return new PageResult<>();
        }
    }



    @Override
    public ReportResp deleteReport(ReportDeleteReq req) throws NoSuchAlgorithmException, KeyStoreException, KeyManagementException {
        RestTemplate restTemplate = getRestTemplateWithoutSANCheck();
        String reportDeleteUrl = ConstantContextHolder.getScaIp() + "openapi/v1/asset/report/delet";
        MultiValueMap<String, Object> body = buildDeleteRequestBody(req);
        HttpHeaders headers = createHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        headers.add("OpenApiUserToken", ConstantContextHolder.getScaToken());
        HttpEntity<MultiValueMap<String, Object>> requestEntity = new HttpEntity<>(body, headers);

        log.info("报告删除接口:" + reportDeleteUrl);
        JSONObject response = restTemplate.postForObject(reportDeleteUrl, requestEntity, JSONObject.class);
        String message = response.getString("message");
        if (message.equals("success")) {
            ReportResp reportResp = new ReportResp();
            log.info("===============报告删除接口成功=================");
            JSONObject data = response.getJSONObject("data");
            setReportDeleteResp(data,response.getString("xmTraceId"),reportResp);

            return reportResp;

        } else {
            log.error("===============报告删除接口失败=================");
            throw new ServiceException(GlobalErrorCodeConstants.INTERNAL_SERVER_ERROR.getCode(),"删除报告失败");
        }
    }



    @Override
    public String downLoadReport(BaseIdReq req) throws NoSuchAlgorithmException, KeyStoreException, KeyManagementException {
        RestTemplate restTemplate = getRestTemplateWithoutSANCheck();
        String reportDeleteUrl = ConstantContextHolder.getScaIp() + "openapi/v1/asset/report/downLoadReport/batch";
        MultiValueMap<String, Object> body = buildDownloadRequestBody(req);
        HttpHeaders headers = createHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        headers.add("OpenApiUserToken", ConstantContextHolder.getScaToken());
        HttpEntity<MultiValueMap<String, Object>> requestEntity = new HttpEntity<>(body, headers);

        log.info("报告下载接口:" + reportDeleteUrl);
        JSONObject response = restTemplate.postForObject(reportDeleteUrl, requestEntity, JSONObject.class);
        String message = response.getString("message");
        if (message.equals("success")) {
            ReportResp reportResp = new ReportResp();
            log.info("===============报告下载接口成功=================");

            return response.getString("xmTraceId");

        } else {
            log.error("===============报告下载接口失败=================");
            throw new ServiceException(GlobalErrorCodeConstants.INTERNAL_SERVER_ERROR.getCode(),"删除报告失败");
        }
    }




    private HttpHeaders createHeaders() {
        HttpHeaders headers = new HttpHeaders();
        headers.add("OpenApiUserToken", ConstantContextHolder.getScaToken());
        return headers;
    }

    private MultiValueMap<String, Object> buildFindRequestBody(BaseIdPageReq req) {
        MultiValueMap<String, Object> body = new LinkedMultiValueMap<>();
        body.add("projectId", req.getProjectId());
        body.add("pageNum", req.getPageNo());
        body.add("pageSize", req.getPageSize());
        return body;
    }

    private MultiValueMap<String, Object> buildDeleteRequestBody(ReportDeleteReq req) {
        MultiValueMap<String, Object> body = new LinkedMultiValueMap<>();
        body.add("endDate", req.getEndDate());
        body.add("name", req.getName());
        body.add("projectIdList", req.getProjectIdList());
        body.add("reportId", req.getReportId());
        body.add("sourceCode", req.getSourceCode());
        body.add("startDate", req.getStartDate());
        body.add("status", req.getStatus());
        body.add("type", req.getType());
        body.add("use", req.getUse());
        return body;
    }

    private MultiValueMap<String, Object> buildDownloadRequestBody(BaseIdReq req) {
        MultiValueMap<String, Object> body = new LinkedMultiValueMap<>();
        body.add("reportIds", req.getReportIds());
        return body;
    }


    private void setReportFindResp(JSONObject data, String xmTraceId, List<ReportResp> respList) {
        data.getJSONArray("records").forEach(item -> {
            JSONObject report = (JSONObject) item;
            ReportResp reportResp = new ReportResp();

            reportResp.setId(report.getInteger("id"));
            reportResp.setCreateDate(report.getString("createDate"));
            reportResp.setCreator(report.getString("creator"));
            reportResp.setDimension(report.getInteger("dimension"));
            reportResp.setMd5(report.getString("md5"));
            reportResp.setName(report.getString("name"));
            reportResp.setProjectName(report.getString("projectName"));
            reportResp.setSerialNumber(report.getString("serialNumber"));
            reportResp.setSource(report.getInteger("source"));
            reportResp.setSourceName(report.getString("sourceName"));
            reportResp.setStatus(report.getInteger("status"));

            reportResp.setXmTraceId(xmTraceId);

            List<Integer> typeList = new ArrayList<>();
            report.getJSONArray("type").forEach(o->{
                typeList.add((Integer) o);
            });
            reportResp.setType(typeList);

            respList.add(reportResp);
        });
    }


    private void setReportDeleteResp(JSONObject data, String xmTraceId, ReportResp reportResp) {
        reportResp.setFail(data.getInteger("fail"));
        reportResp.setSuccess(data.getInteger("success"));
        reportResp.setXmTraceId(xmTraceId);
    }


}
